@@ -1,14 +1,13 @@
-import concurrent
+import collections
 import copy
+import logging
 import os.path
 import os
 import re
-from concurrent.futures._base import ALL_COMPLETED
-from concurrent.futures.thread import ThreadPoolExecutor
-import logging
 import json
 import time
-from typing import List, Dict, Optional
+from queue import Queue
+from typing import List, Dict, Optional, Deque
 
 import urllib.parse
 from botocore.exceptions import ClientError, EndpointConnectionError
@@ -19,9 +18,9 @@ from dragoneye.cloud_scanner.base_cloud_scanner import BaseCloudScanner
 from dragoneye.utils.app_logger import logger
 from dragoneye.utils.misc_utils import get_dynamic_values_from_files, custom_serializer, make_directory, init_directory, load_yaml, snakecase, \
     elapsed_time
+from dragoneye.utils.threading_utils import execute_parallel_functions_in_threads, ThreadedFunctionData
 
 MAX_RETRIES = 3
-MAX_WORKER = 10
 
 
 class AwsScanner(BaseCloudScanner):
@@ -39,19 +38,32 @@ class AwsScanner(BaseCloudScanner):
             "cloudfront",
             "organizations",
         ]
+        self.scan_commands = None
+        self.default_region = settings.default_region or self.session.region_name
+        if self.default_region is None:
+            raise ValueError('Default region cannot be empty. '
+                             'You must specify the default region or set the AWS_DEFAULT_REGION environment variable')
         logging.getLogger("botocore").setLevel(logging.WARN)
 
     @elapsed_time('Scanning AWS live environment took {} seconds')
     def scan(self) -> str:
+        self.scan_commands = load_yaml(self.settings.commands_path)
         account_data_dir = init_directory(self.settings.output_path, self.settings.account_name, self.settings.clean)
         region_dict_list = self._create_regions_file_structure(account_data_dir)
 
-        summary = []
-        executor: ThreadPoolExecutor = ThreadPoolExecutor(max_workers=MAX_WORKER)
+        summary: Queue = Queue()
+        tasks: List[ThreadedFunctionData] = []
 
         for region in region_dict_list:
-            executor.submit(self._scan_region_data, region, account_data_dir, summary)
-        executor.shutdown(True)
+            tasks.append(ThreadedFunctionData(
+                self._scan_region_data,
+                (region, account_data_dir, summary),
+                'An unknown exception has occurred'
+            ))
+
+        deque_tasks: Deque[List[ThreadedFunctionData]] = collections.deque()
+        deque_tasks.append(tasks)
+        execute_parallel_functions_in_threads(deque_tasks, len(region_dict_list))
 
         self._print_summary(summary)
 
@@ -76,11 +88,11 @@ class AwsScanner(BaseCloudScanner):
         if len(self.settings.regions_filter) > 0:
             regions_filter = self.settings.regions_filter.lower().split(",")
             # Force include of default region -- seems to be required
-            if self.settings.default_region not in regions_filter:
-                regions_filter.append(self.settings.default_region)
+            if self.default_region not in regions_filter:
+                regions_filter.append(self.default_region)
 
         logger.info("* Getting region names")
-        ec2 = self.session.client("ec2")
+        ec2 = self.session.client("ec2", region_name=self.default_region)
         region_list = ec2.describe_regions()
 
         if regions_filter is not None:
@@ -90,14 +102,14 @@ class AwsScanner(BaseCloudScanner):
         return region_list
 
     @staticmethod
-    def _print_summary(summary):
+    def _print_summary(summary: Queue):
         logger.info("--------------------------------------------------------------------")
         failures = []
-        for call_summary in summary:
+        for call_summary in summary.queue:
             if "exception" in call_summary:
                 failures.append(call_summary)
 
-        logger.info("Summary: {} APIs called. {} errors".format(len(summary), len(failures)))
+        logger.info("Summary: {} APIs called. {} errors".format(len(summary.queue), len(failures)))
         if len(failures) > 0:
             logger.warning("Failures:")
             for call_summary in failures:
@@ -138,7 +150,7 @@ class AwsScanner(BaseCloudScanner):
         return urllib.parse.quote_plus(filename)
 
     @staticmethod
-    def _get_and_save_data(output_file, handler, method_to_call, parameters, checks, summary):
+    def _get_and_save_data(output_file, handler, method_to_call, parameters, checks, region, summary: Queue):
         """
         Calls the AWS API function and downloads the data
 
@@ -156,6 +168,7 @@ class AwsScanner(BaseCloudScanner):
             "service": handler.meta.service_model.service_name,
             "action": method_to_call,
             "parameters": parameters,
+            "region": region
         }
 
         data = AwsScanner._get_data(output_file, handler, method_to_call, parameters, checks, call_summary)
@@ -163,16 +176,17 @@ class AwsScanner(BaseCloudScanner):
         AwsScanner._save_results_to_file(output_file, data)
 
         logger.info("finished call for {}".format(output_file))
-        summary.append(call_summary)
+        summary.put_nowait(call_summary)
 
     @staticmethod
     def _get_data(output_file, handler, method_to_call, parameters, checks, call_summary):
         logger.info("  Making call for {}".format(output_file))
+        data = None
         try:
             for retries in range(MAX_RETRIES):
                 data = AwsScanner._call_boto_function(output_file, handler, method_to_call, parameters)
                 if not checks or AwsScanner._is_data_passing_check(data, checks):
-                    pass
+                    break
                 elif retries == MAX_RETRIES - 1:
                     raise Exception(
                         "One of the following checks has repeatedly failed: {}".format(
@@ -245,6 +259,20 @@ class AwsScanner(BaseCloudScanner):
                 logger.warning("  - Denied, which should mean this KMS has restricted access")
             elif "AWSOrganizationsNotInUseException" in str(ex):
                 logger.warning(' - Your account is not a member of an organization.')
+            elif (
+                    "EntityNotFoundException" in str(ex)
+                    and call_summary["service"] == "glue"
+                    and call_summary["action"] == "get_resource_policy"
+            ):
+                logger.warning(f' - Glue policy does not exist on region {call_summary["region"]}')
+            elif (
+                    "NoSuchEntity" in str(ex)
+            ):
+                logger.warning(f"  - {str(ex)}")
+            elif (
+                    "NoSuchAccessPointPolicy" in str(ex)
+            ):
+                logger.warning(f"  - {str(ex)}")
             else:
                 logger.warning(f"ClientError {retries}: {ex}")
                 call_summary["exception"] = ex
@@ -303,11 +331,10 @@ class AwsScanner(BaseCloudScanner):
                     json.dumps(data, indent=4, sort_keys=True, default=custom_serializer)
                 )
 
-    def _run_scan_commands(self, region, runner, account_dir, summary):
-        executor: ThreadPoolExecutor = ThreadPoolExecutor(max_workers=20)
-        tasks = []
+    def _run_scan_commands(self, region, runner, account_dir, summary: Queue):
         region = copy.deepcopy(region)
         runner = copy.deepcopy(runner)
+        region_name = region["RegionName"]
         logger.info(
             "* Getting {}:{}:{} info".format(region["RegionName"], runner["Service"], runner["Request"])
         )
@@ -326,50 +353,73 @@ class AwsScanner(BaseCloudScanner):
         parameter_keys = set()
         param_groups = self._get_parameter_group(runner, account_dir, region, parameter_keys)
 
+        tasks: List[ThreadedFunctionData] = []
+
         if runner.get("Parameters"):
             make_directory(filepath)
             for param_group in param_groups:
                 if set(param_group.keys()) != parameter_keys:
                     continue
-                file_name = urllib.parse.quote_plus('_'.join([f'{k}-{v}' for k, v in param_group.items()]))
+                unparsed_file_name = '_'.join([f'{k}-{v}' if not isinstance(v, list) else k for k, v in param_group.items()])
+                file_name = urllib.parse.quote_plus(unparsed_file_name)
                 output_file = f"{filepath}/{file_name}.json"
-                tasks.append(executor.submit(AwsScanner._get_and_save_data,
-                                             output_file,
-                                             handler,
-                                             method_to_call,
-                                             param_group,
-                                             runner.get("Check", None),
-                                             summary,
-                                             ))
+                tasks.append(ThreadedFunctionData(
+                    AwsScanner._get_and_save_data,
+                    (output_file,
+                     handler,
+                     method_to_call,
+                     param_group,
+                     runner.get("Check", None),
+                     region_name,
+                     summary),
+                    'exception on command {}'.format(runner),
+                    'timeout on command {}'.format(runner)))
         else:
-            filepath = filepath + ".json"
-            tasks.append(executor.submit(AwsScanner._get_and_save_data,
-                                         filepath,
-                                         handler,
-                                         method_to_call,
-                                         {},
-                                         runner.get("Check", None),
-                                         summary,
-                                         ))
-        concurrent.futures.wait(tasks, timeout=self.settings.command_timeout, return_when=ALL_COMPLETED)
-        timeout_tasks = [task for task in tasks if task.running()]
-        if timeout_tasks:
-            logging.exception('timeout command {}'.format(runner))
-        for timeout_task in timeout_tasks:
-            timeout_task.cancel()
+            output_file = filepath + ".json"
+            tasks.append(ThreadedFunctionData(
+                AwsScanner._get_and_save_data,
+                (output_file,
+                 handler,
+                 method_to_call,
+                 {},
+                 runner.get("Check", None),
+                 region_name,
+                 summary), 'exception on command {}'.format(runner), 'timeout on command {}'.format(runner)))
 
-    def _scan_region_data(self, region, account_dir, summary):
-        scan_commands = load_yaml(self.settings.commands_path)
-        dependable_commands = [command for command in scan_commands if command.get("Parameters", False)]
-        non_dependable_commands = [command for command in scan_commands if not command.get("Parameters", False)]
+        deque_tasks: Deque[List[ThreadedFunctionData]] = collections.deque()
+        deque_tasks.append(tasks)
+        max_workers = 10 if runner['Service'] == 'lambda' else 20
+        execute_parallel_functions_in_threads(deque_tasks, max_workers, self.settings.command_timeout)
 
-        executor: ThreadPoolExecutor = ThreadPoolExecutor(max_workers=20)
-        for command in non_dependable_commands:
-            executor.submit(self._run_scan_commands, region, command, account_dir, summary)
-        executor.shutdown(True)
+    def _scan_region_data(self, region, account_dir, summary: Queue):
+        dependable_commands = []
+        non_dependable_commands = []
+        for command in self.scan_commands:
+            if "Parameters" in command:
+                dependable_commands.append(command)
+            else:
+                non_dependable_commands.append(command)
 
-        for command in dependable_commands:
-            self._run_scan_commands(region, command, account_dir, summary)
+        non_dependable_tasks: List[ThreadedFunctionData] = []
+        dependable_tasks: List[ThreadedFunctionData] = []
+        deque_tasks: Deque[List[ThreadedFunctionData]] = collections.deque()
+
+        for non_dependable_command in non_dependable_commands:
+            non_dependable_tasks.append(ThreadedFunctionData(
+                self._run_scan_commands,
+                (region, non_dependable_command, account_dir, summary),
+                'exception on command {}'.format(non_dependable_command)))
+
+        deque_tasks.append(non_dependable_tasks)
+
+        for dependable_command in dependable_commands:
+            dependable_tasks.append(ThreadedFunctionData(
+                self._run_scan_commands,
+                (region, dependable_command, account_dir, summary),
+                'exception on command {}'.format(dependable_command)))
+
+        deque_tasks.append(dependable_tasks)
+        execute_parallel_functions_in_threads(deque_tasks, 20)
 
     @staticmethod
     def _get_call_parameters(call_parameters: dict, parameters_def: list) -> List[dict]:
@@ -435,18 +485,18 @@ class AwsScanner(BaseCloudScanner):
                 for val in values:
                     param_groups.append({name: val})
                 return param_groups
-        cached_values = {}
+
         result_param_groups = []
         for param_group in param_groups:
             real_value = value
             for key in depends_on_keys:
                 real_value = real_value.replace(f'{{{{{key}}}}}', param_group[key])
-            cached_values[real_value] = cached_values.get(real_value, get_dynamic_values_from_files(real_value, region_account_dir))
+            dynamic_params_list: list = get_dynamic_values_from_files(real_value, region_account_dir)
             if group:
-                param_group[name] = cached_values[real_value]
+                param_group[name] = dynamic_params_list
                 result_param_groups.append(param_group)
             else:
-                for cached_value in cached_values[real_value]:
+                for cached_value in dynamic_params_list:
                     clone_param_group = copy.deepcopy(param_group)
                     clone_param_group[name] = cached_value
                     result_param_groups.append(clone_param_group)
@@ -454,7 +504,7 @@ class AwsScanner(BaseCloudScanner):
 
     def _should_run_command_on_region(self, runner: dict, region_dict: dict) -> bool:
         if runner["Service"] in self.universal_services:
-            if region_dict["RegionName"] != self.settings.default_region:
+            if region_dict["RegionName"] != self.default_region:
                 return False
         elif runner["Service"] != 'eks' and region_dict["RegionName"] not in self.session.get_available_regions(runner["Service"]):
             logger.info("Skipping region {}, as {} does not exist there"
