@@ -1,6 +1,6 @@
+import collections
 import os
-from concurrent.futures.thread import ThreadPoolExecutor
-from typing import List
+from typing import List, Deque
 
 import json
 
@@ -11,6 +11,7 @@ from dragoneye.cloud_scanner.base_cloud_scanner import BaseCloudScanner
 from dragoneye.utils.misc_utils import elapsed_time, invoke_get_request, init_directory, load_yaml, get_dynamic_values_from_files, \
     custom_serializer
 from dragoneye.utils.app_logger import logger
+from dragoneye.utils.threading_utils import ThreadedFunctionData, execute_parallel_functions_in_threads
 
 
 class AzureScanner(BaseCloudScanner):
@@ -33,32 +34,53 @@ class AzureScanner(BaseCloudScanner):
         scan_commands = load_yaml(settings.commands_path)
         resource_groups = self._get_resource_groups(headers, subscription_id, account_data_dir)
 
-        dependable_commands = [command for command in scan_commands if command.get("Parameters", False)]
-        non_dependable_commands = [command for command in scan_commands if not command.get("Parameters", False)]
+        dependable_commands = []
+        non_dependable_commands = []
+        for command in scan_commands:
+            if "Parameters" in command:
+                dependable_commands.append(command)
+            else:
+                non_dependable_commands.append(command)
+
+        non_dependable_tasks: List[ThreadedFunctionData] = []
+        dependable_tasks: List[ThreadedFunctionData] = []
+        deque_tasks: Deque[List[ThreadedFunctionData]] = collections.deque()
 
-        executor: ThreadPoolExecutor = ThreadPoolExecutor(max_workers=20)
         for non_dependable_command in non_dependable_commands:
-            executor.submit(self._execute_scan_commands, non_dependable_command, subscription_id, headers, account_data_dir, resource_groups)
-        executor.shutdown(True)
+            non_dependable_tasks.append(ThreadedFunctionData(
+                self._execute_scan_commands,
+                (non_dependable_command, subscription_id, headers, account_data_dir, resource_groups),
+                'exception on command {}'.format(non_dependable_command)))
+
+        deque_tasks.append(non_dependable_tasks)
 
         for dependable_command in dependable_commands:
-            self._execute_scan_commands(dependable_command, subscription_id, headers, account_data_dir, resource_groups)
+            dependable_tasks.append(ThreadedFunctionData(
+                self._execute_scan_commands,
+                (dependable_command, subscription_id, headers, account_data_dir, resource_groups),
+                'exception on command {}'.format(dependable_command)))
+
+        deque_tasks.append(dependable_tasks)
+        execute_parallel_functions_in_threads(deque_tasks, 20)
 
         return os.path.abspath(os.path.join(account_data_dir, '..'))
 
     def _execute_scan_commands(self, scan_command: dict, subscription_id: str, headers: dict,
                                account_data_dir: str, resource_groups: List[str]) -> None:
-        output_file = self._get_result_file_path(account_data_dir, scan_command['Name'])
-        if os.path.isfile(output_file):
-            # Data already scanned, so skip
-            logger.warning("  Response already present at {}".format(output_file))
-            return
+        try:
+            output_file = self._get_result_file_path(account_data_dir, scan_command['Name'])
+            if os.path.isfile(output_file):
+                # Data already scanned, so skip
+                logger.warning('  Response already present at {}'.format(output_file))
+                return
 
-        request = scan_command['Request']
-        parameters = scan_command.get('Parameters', [])
-        url = request.replace('{subscriptionId}', subscription_id)
-        results = AzureScanner._get_results(url, headers, parameters, account_data_dir, resource_groups)
-        self._save_result(results, output_file)
+            request = scan_command['Request']
+            parameters = scan_command.get('Parameters', [])
+            url = request.replace('{subscriptionId}', subscription_id)
+            results = AzureScanner._get_results(url, headers, parameters, account_data_dir, resource_groups)
+            self._save_result(results, output_file)
+        except Exception as ex:
+            logger.exception('exception on command {}'.format(scan_command), exc_info=ex)
 
     def _save_result(self, result: dict, filepath: str) -> None:
         self._add_resource_group(result)
